package com.javacodegeeks;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.dbcp2.DelegatingPreparedStatement;
import org.apache.commons.dbcp2.PStmtKey;
import org.apache.commons.dbcp2.PoolingConnection;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;
import org.apache.commons.pool2.impl.GenericKeyedObjectPoolConfig;

/**
 * Create Table Query for MS SQL Server CREATE TABLE Users(id varchar(255),name
 * varchar(255),passion varchar(255));
 *
 */

public class PoolingConnectionExample {

	private static PoolingConnection poolingConnection;
	static Connection connection = null;

	public static void main(String... s) throws SQLException,
			ClassNotFoundException, InstantiationException,
			IllegalAccessException {
		createPoolingConnection();
		insertQuery("1", "tom", "chasing jerry");
		insertQuery("2", "jerry", "eating cheese");
		insertQuery("3", "jcg", "helping you learn faster");
		selectQuery();
		connection.close();

	}

	private static void createPoolingConnection()
			throws ClassNotFoundException, SQLException {
		Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
		connection = DriverManager
				.getConnection("jdbc:sqlserver://localhost;database=TestDB;user=;password=");

		poolingConnection = new PoolingConnection(connection);

		GenericKeyedObjectPoolConfig config = new GenericKeyedObjectPoolConfig();
		config.setMaxTotalPerKey(-1);
		config.setBlockWhenExhausted(false);
		config.setMaxWaitMillis(0);
		config.setMaxIdlePerKey(1);
		config.setMaxTotalPerKey(-1);
		config.setBlockWhenExhausted(false);
		config.setMaxWaitMillis(0);
		config.setMaxIdlePerKey(1);
		config.setMaxTotal(10);

		KeyedObjectPool<PStmtKey, DelegatingPreparedStatement> stmtPool = new GenericKeyedObjectPool<>(
				poolingConnection, config);

		poolingConnection.setStatementPool(stmtPool);
	}

	private static void insertQuery(String id, String name, String passion)
			throws SQLException {
		String sql = "INSERT INTO Users VALUES(?,?,?)";
		PreparedStatement statement = poolingConnection.prepareStatement(sql);
		statement.setString(1, id);
		statement.setString(2, name);
		statement.setString(3, passion);
		statement.execute();
		statement.close();

	}

	private static void selectQuery() throws SQLException {
		PreparedStatement statement = null;
		System.out
				.println("*********************Selecting data************************");
		System.out.println();

		try {
			statement = poolingConnection
					.prepareStatement("SELECT * FROM Users");
			ResultSet rs = statement.executeQuery();

			while (rs.next()) {
				String s = String.format("ID:%s Username:%s Passion:%s",
						rs.getString("id"), rs.getString("name"),
						rs.getString("passion"));
				System.out.println(s);
				System.out.println();
			}
			System.out.println("");
			System.out
					.println("*******************************************************");
		} catch (Exception e) {
			System.out.println(e.getMessage());
		} finally {
			if (statement != null)
				statement.close();
			if (connection != null)
				connection.close();
		}
	}
}
